#######################################################################
#
# NCM component for rgmaserver daemon
#
# Charles Loomis <charles.loomis@cern.ch>
# Version 1.0.5 : 11/04/05 19:18
#
# ** Generated file : do not edit **
#
#######################################################################

package NCM::Component::rgmaserver;

use strict;
use NCM::Component;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;
use NCM::Check;

use EDG::WP4::CCM::Element;

use File::Path;
use File::Copy;

local(*DTA);


##########################################################################
sub Configure($$@) {
##########################################################################
    
    my ($self, $config) = @_;

    # Define paths for convenience. 
    my $base = "/software/components/rgmaserver";

    # Get the RGMA home area. 
    my $rgmaHome = "/opt/edg";
    if ($config->elementExists("$base/rgmaHome")) {
	$rgmaHome = $config->getValue("$base/rgmaHome");
    }

    # Get the tomcat installation area. 
    my $catalinaHome = "/var/tomcat4";
    if ($config->elementExists("$base/catalinaHome")) {
	$catalinaHome = $config->getValue("$base/catalinaHome");
    }

    # Modify the environment with these locations. 
    $ENV{RGMA_HOME} = $rgmaHome;
    $ENV{CATALINA_HOME} = $catalinaHome;

    # Clean out any old deployments of R-GMA.
    if (-e "$catalinaHome/webapps/R-GMA") {
	rmtree("$catalinaHome/webapps/R-GMA",0,0);
    }

    # If the webapps area doesn't exist, complain and exit.
    if (! -d "$catalinaHome/webapps") {
	$self->error("tomcat webapps ($catalinaHome/webapps) doesn't exist");
	return 1;
    }

    # Same for the web archive. 
    if (! -e "$rgmaHome/share/webapps/R-GMA.war") {
	$self->error("R-GMA web archive doesn't exist");
	return 1;
    }

    # Copy the web archive into place.
    copy "$rgmaHome/share/webapps/R-GMA.war", 
	"$catalinaHome/webapps/R-GMA.war";

    # Get the mon and registry hosts.
    my $monHost = $config->getValue("$base/monHost");
    my $regHost = $config->getValue("$base/regHost");

    # Create the command.
    my $flag = ($monHost eq $regHost) ? 'yes' : 'no' ; 
    my @cmd = ("$rgmaHome/share/rgma/scripts/rgma-server-setup.py", 
	       "--schema=$flag", 
	       "--registry=$flag",
	       "--browser=yes");

    # Run it.
    $self->info("RUNNING: " . join(' ',@cmd));
    system(@cmd);
    if ($?) {
	$self->error("rgma-server-setup.py failed: " . ($? >> 8));
	return 1;
    }

    # Get the MySQL password. 
    my $mysqlPwd = $config->getValue("$base/mysqlPwd");

    # Assume that the MySQL server is running and setup the user for
    # R-GMA.
    my $cmdline = "mysql -u root --pass=\"$mysqlPwd\" " . 
	"< $rgmaHome/etc/rgma-server/rgma_sql_conf.sql";
    `$cmdline`;
    $self->error("error configuring mysql for R-GMA") if ($?);

    # Hack the tomcat server configuration to increase the number
    # of parallel threads. 
    my $fname = "$catalinaHome/conf/server.xml";
    my $contents = '';
    open XML, "<", $fname;
    while (<XML>) {
	s/maxProcessors="\d+"/maxProcessors="1000"/g;
	$contents .= $_;
    }
    close XML;
    open XML, ">", $fname;
    print XML $contents;
    close XML;

    # Get the ram size for the system.  This is probably very 
    # OS-dependent.  Probably should be done through the quattor
    # configuration.
    my $javaMemory = 256;
    my $dummy;
    my $output = `free -m`;
    open TEMP, "free -m |";
    while (<TEMP>) {
	chomp;
	($dummy, $javaMemory) = split /\s+/ if (m/Mem:/);
    }
    close TEMP;
    $javaMemory = int($javaMemory/2);

    # Hack also the tomcat startup script to increase the java memory. 
    $fname = "/etc/tomcat4/tomcat4.conf";
    $contents = '';
    my $found = 0;
    open TEMP, "<", $fname;
    while (<TEMP>) {
	if (m/JAVA_OPTS/) {
	    s/-Xmx\d+M/-Xmx${javaMemory}M/;
	    $found = 1;
	}
	$contents .= $_;
    }
    $contents .= "JAVA_OPTS=\"-Xmx${javaMemory}M\"\n" unless ($found);
    close TEMP;

    open TEMP, ">", $fname;
    print TEMP $contents;
    close TEMP;

    # Get the site name.
    my $siteName = $config->getValue("$base/siteName");

    # Create the service tool configuration file. 
    open TEMP, ">", "$rgmaHome/etc/rgma-servicetool/servicetool.conf";
    print TEMP "site=$siteName\n";
    close TEMP;

    # Restart the services: tomcat and servicetool.
    `/etc/rc.d/init.d/tomcat4 restart`;
    $self->error("problem restarting tomcat") if ($?);
    `/etc/rc.d/init.d/edg-rgma-servicetool restart`;
    $self->error("problem restarting edg-rgma-servicetool") if ($?);

    return 1;
}

1;      # Required for PERL modules
